@@ -16,7 +16,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Convert Gettext PO localization files to Java/Mozilla .properties files.
+"""
+Convert Gettext PO localization files to Java/Mozilla .properties files.
 
 See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/commands/prop2po.html
 for examples and usage instructions.
@@ -32,7 +33,7 @@ eol = "\n"
 
 
 def applytranslation(key, propunit, inunit, mixedkeys):
-    """applies the translation for key in the po unit to the prop unit"""
+    """Applies the translation for key in the po unit to the prop unit."""
     # this converts the po-style string to a prop-style string
     value = inunit.target
     # handle mixed keys
@@ -45,7 +46,7 @@ def applytranslation(key, propunit, inunit, mixedkeys):
             if key.endswith(akeysuffix) and key in mixedkeys:
                 label, value = accesskey.extract(value)
                 if not value:
-                    warnings.warn("Could not find accesskey for %s" % key)
+                    warnings.warn(f"Could not find accesskey for {key}")
                     # Use the source language accesskey
                     label, value = accesskey.extract(inunit.source)
                 else:
@@ -165,7 +166,7 @@ class reprop:
                 if category != "":
                     new_location = f"{location}[{category}]"
                 else:
-                    new_location = "%s" % (location)
+                    new_location = f"{location}"
                 new_unit.addlocation(new_location)
                 new_unit.target = text
                 self.inputstore.locationindex[new_location] = new_unit
@@ -190,7 +191,7 @@ class reprop:
                 self.inmultilinemsgid = True
             if delimiter_pos == -1:
                 key = self.personality.key_strip(line)
-                delimiter = " %s " % self.personality.delimiters[0]
+                delimiter = f" {self.personality.delimiters[0]} "
             else:
                 key = self.personality.key_strip(line[:delimiter_pos])
                 # Calculate space around the equal sign
@@ -198,8 +199,7 @@ class reprop:
                 postspacestart = len(line[delimiter_pos + 1 :])
                 postspaceend = len(line[delimiter_pos + 1 :].lstrip())
                 postspace = line[
-                    delimiter_pos
-                    + 1 : delimiter_pos
+                    delimiter_pos + 1 : delimiter_pos
                     + (postspacestart - postspaceend)
                     + 1
                 ]
@@ -226,19 +226,9 @@ class reprop:
                     self.inecho = False
                     assert isinstance(value, str)
                     returnline = "%(key)s%(del)s%(value)s%(term)s%(eol)s" % {
-                        "key": "%s%s%s"
-                        % (
-                            self.personality.key_wrap_char,
-                            key,
-                            self.personality.key_wrap_char,
-                        ),
+                        "key": f"{self.personality.key_wrap_char}{key}{self.personality.key_wrap_char}",
                         "del": delimiter if delimiter_pos != -1 or value else "",
-                        "value": "%s%s%s"
-                        % (
-                            self.personality.value_wrap_char,
-                            self.personality.encode(value),
-                            self.personality.value_wrap_char,
-                        ),
+                        "value": f"{self.personality.value_wrap_char}{self.personality.encode(value)}{self.personality.value_wrap_char}",
                         "term": self.personality.pair_terminator,
                         "eol": eol,
                     }
@@ -259,7 +249,7 @@ def convertstrings(
     outputthreshold=None,
     remove_untranslated=False,
 ):
-    """.strings specific convertor function"""
+    """.strings specific convertor function."""
     return convertprop(
         inputfile,
         outputfile,
@@ -280,7 +270,7 @@ def convertmozillaprop(
     remove_untranslated=False,
     outputthreshold=None,
 ):
-    """Mozilla specific convertor function"""
+    """Mozilla specific convertor function."""
     return convertprop(
         inputfile,
         outputfile,
@@ -309,11 +299,9 @@ def convertprop(
 
     if templatefile is None:
         raise ValueError("must have template file for properties files")
-        # convertor = po2prop()
-    else:
-        convertor = reprop(
-            templatefile, inputstore, personality, encoding, remove_untranslated
-        )
+    convertor = reprop(
+        templatefile, inputstore, personality, encoding, remove_untranslated
+    )
     outputprop = convertor.convertstore(includefuzzy)
     outputfile.write(outputprop)
     return True
@@ -338,8 +326,9 @@ def main(argv=None):
         default=properties.default_dialect,
         type="choice",
         choices=list(properties.dialects),
-        help="override the input file format: %s (for .properties files, default: %s)"
-        % (", ".join(properties.dialects), properties.default_dialect),
+        help="override the input file format: {} (for .properties files, default: {})".format(
+            ", ".join(properties.dialects), properties.default_dialect
+        ),
         metavar="TYPE",
     )
     parser.add_option(
