@@ -98,9 +98,7 @@ def isvalidaccelerator(accelerator, acceptlist=None):
         return False
     if acceptlist is not None:
         acceptlist = data.normalize(acceptlist)
-        if accelerator in acceptlist:
-            return True
-        return False
+        return accelerator in acceptlist
     # Old code path - ensures that we don't get a large number of
     # regressions
     accelerator = accelerator.replace("_", "")
@@ -241,7 +239,7 @@ def getnumbers(str1):
         if chr1.isdigit():
             innumber = True
         elif innumber:
-            if chr1 not in (".", degreesign):
+            if chr1 not in {".", degreesign}:
                 innumber = False
                 if lastnumber:
                     numbers.append(lastnumber)
@@ -262,6 +260,8 @@ def getnumbers(str1):
     return numbers
 
 
+_email_re = re.compile(r"[\w\.\-]+@[\w\.\-]+")
+
 _function_re = re.compile(
     r"""((?:
     [\w\.]+              # function or module name - any alpha-numeric character, _, or .
@@ -289,7 +289,7 @@ def getfunctions(str1):
 
 def getemails(str1):
     """Returns the email addresses that are in a string."""
-    return re.findall(r"[\w\.\-]+@[\w\.\-]+", str1)
+    return _email_re.findall(str1)
 
 
 def geturls(str1):
