@@ -1,4 +1,5 @@
-"""selector - WSGI delegation based on URL path and method.
+"""
+selector - WSGI delegation based on URL path and method.
 
 (See the docstring of selector.Selector.)
 
@@ -15,24 +16,23 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to 
-the Free Software Foundation, Inc., 51 Franklin Street, 
+License along with this library; if not, write to
+the Free Software Foundation, Inc., 51 Franklin Street,
 Fifth Floor, Boston, MA  02110-1301  USA
 
 Luke Arno can be found at http://lukearno.com/
 
 """
 
+import contextlib
 import re
 from itertools import starmap
 from wsgiref.util import shift_path_info
 
-try:
+# resolver not essential for basic featurs
+# FIXME: this library is overkill, simplify
+with contextlib.suppress(ImportError):
     from resolver import resolve
-except ImportError:
-    # resolver not essential for basic featurs
-    # FIXME: this library is overkill, simplify
-    pass
 
 
 class MappingFileError(Exception):
@@ -69,7 +69,8 @@ def not_found(environ, start_response):
 
 
 class Selector:
-    """WSGI middleware for URL paths and HTTP method based delegation.
+    """
+    WSGI middleware for URL paths and HTTP method based delegation.
 
     See http://lukearno.com/projects/selector/
 
@@ -105,7 +106,8 @@ class Selector:
         self.consume_path = consume_path
 
     def slurp(self, mappings, prefix=None, parser=None, wrap=None):
-        """Slurp in a whole list (or iterable) of mappings.
+        """
+        Slurp in a whole list (or iterable) of mappings.
 
         Prefix and parser args will override self.parser and self.args
         for the given mappings.
@@ -128,7 +130,8 @@ class Selector:
             self.prefix = oldprefix
 
     def add(self, path, method_dict=None, prefix=None, **http_methods):
-        """Add a mapping.
+        """
+        Add a mapping.
 
         HTTP methods can be specified in a dict or using kwargs,
         but kwargs will override if both are given.
@@ -163,7 +166,7 @@ class Selector:
         environ["selector.vars"] = dict(named)
         for k in named:
             if k.isdigit():
-                unnamed.append((k, named.pop(k)))
+                unnamed.append((k, named.pop(k)))  # noqa: PERF401
         unnamed.sort()
         unnamed = [v for k, v in unnamed]
         cur_unnamed, cur_named = environ.get("wsgiorg.routing_args", ([], {}))
@@ -190,19 +193,19 @@ class Selector:
                         methods,
                         match.group(0),
                     )
-                elif "_ANY_" in method_dict:
+                if "_ANY_" in method_dict:
                     return (
                         method_dict["_ANY_"],
                         match.groupdict(),
                         methods,
                         match.group(0),
                     )
-                else:
-                    return self.status405, {}, methods, ""
+                return self.status405, {}, methods, ""
         return self.status404, {}, [], ""
 
     def slurp_file(self, the_file, prefix=None, parser=None, wrap=None):
-        """Read mappings from a simple text file.
+        """
+        Read mappings from a simple text file.
 
         Format looks like this::
 
@@ -261,19 +264,18 @@ class Selector:
             self.prefix = oldprefix
 
     def _parse_line(self, line, path, methods):
-        """Parse one line of a mapping file.
+        """
+        Parse one line of a mapping file.
 
         This method is for the use of selector.slurp_file.
         """
         if not line.strip() or line.strip()[0] == "#":
             pass
         elif not line.strip() or line.strip()[0] == "@":
-            #
             if path and methods:
                 self.add(path, methods)
             path = line.strip()
             methods = {}
-            #
             parts = line.strip()[1:].split(" ", 1)
             if len(parts) == 2:
                 directive, rest = parts
@@ -298,7 +300,8 @@ class Selector:
 
 
 class SimpleParser:
-    r"""Callable to turn path expressions into regexes with named groups.
+    r"""
+    Callable to turn path expressions into regexes with named groups.
 
     For instance ``"/hello/{name}"`` becomes ``r"^\/hello\/(?P<name>[^\^.]+)$"``
 
@@ -352,26 +355,28 @@ class SimpleParser:
         else:
             pattern = self.patterns[self.default_pattern]
         if name == "":
-            name = "__pos%s" % self._pos
+            name = f"__pos{self._pos}"
             self._pos += 1
         return f"(?P<{name}>{pattern})"
 
     @staticmethod
     def lastly(regex):
-        """Process the result of __call__ right before it returns.
+        """
+        Process the result of __call__ right before it returns.
 
         Adds the ^ and the $ to the beginning and the end, respectively.
         """
-        return "^%s$" % regex
+        return f"^{regex}$"
 
     @staticmethod
     def openended(regex):
-        """Process the result of ``__call__`` right before it returns.
+        """
+        Process the result of ``__call__`` right before it returns.
 
         Adds the ^ to the beginning but no $ to the end.
         Called as a special alternative to lastly.
         """
-        return "^%s" % regex
+        return f"^{regex}"
 
     def outermost_optionals_split(self, text):
         """Split out optional portions by outermost matching delims."""
@@ -406,7 +411,7 @@ class SimpleParser:
         if self.ostart in text:
             parts = self.outermost_optionals_split(text)
             parts = map(self.parse, parts)
-            parts[1::2] = ["(%s)?" % p for p in parts[1::2]]
+            parts[1::2] = [f"({p})?" for p in parts[1::2]]
         else:
             parts = [part.split(self.end) for part in text.split(self.start)]
             parts = [y for x in parts for y in x]
@@ -419,10 +424,10 @@ class SimpleParser:
         self._pos = 0
         if url_pattern.endswith("|"):
             return self.openended(self.parse(url_pattern[:-1]))
-        else:
-            return self.lastly(self.parse(url_pattern))
+        return self.lastly(self.parse(url_pattern))
 
 
+# TODO: Is this needed?
 class EnvironDispatcher:
     """Dispatch based on list of rules."""
 
@@ -431,13 +436,15 @@ class EnvironDispatcher:
         self.rules = rules
 
     def __call__(self, environ, start_response):
-        """Call the first app whose predicate is true.
+        """
+        Call the first app whose predicate is true.
 
         Each predicate is passes the environ to evaluate.
         """
         for predicate, app in self.rules:
             if predicate(environ):
                 return app(environ, start_response)
+        return None
 
 
 class MiddlewareComposer:
@@ -449,7 +456,8 @@ class MiddlewareComposer:
         self.rules = rules
 
     def __call__(self, environ, start_response):
-        """Apply each middleware whose predicate is true.
+        """
+        Apply each middleware whose predicate is true.
 
         Each predicate is passes the environ to evaluate.
 
@@ -484,7 +492,8 @@ class Naked:
     _exposed = True
 
     def _is_exposed(self, obj):
-        """Determine if obj should be exposed.
+        """
+        Determine if obj should be exposed.
 
         If ``self._expose_all`` is True, always return True.
         Otherwise, look at obj._exposed.
@@ -494,14 +503,13 @@ class Naked:
     def __call__(self, environ, start_response):
         """Dispatch to the method named by the next bit of PATH_INFO."""
         name = shift_path_info(
-            dict(SCRIPT_NAME=environ["SCRIPT_NAME"], PATH_INFO=environ["PATH_INFO"])
+            {"SCRIPT_NAME": environ["SCRIPT_NAME"], "PATH_INFO": environ["PATH_INFO"]}
         )
         callable = getattr(self, name or "index", None)
         if callable is not None and self._is_exposed(callable):
             shift_path_info(environ)
             return callable(environ, start_response)
-        else:
-            return self._not_found(environ, start_response)
+        return self._not_found(environ, start_response)
 
 
 class ByMethod:
@@ -518,9 +526,10 @@ class ByMethod:
 
 
 def pliant(func):
-    """Decorate an unbound wsgi callable taking args from
-    ``wsgiorg.routing_args``
-    ::
+    """
+    Decorate an unbound wsgi callable taking args from ``wsgiorg.routing_args``.
+
+    .. code-block:: python
 
         @pliant
         def app(environ, start_response, arg1, arg2, foo='bar'):
@@ -538,9 +547,10 @@ def pliant(func):
 
 
 def opliant(meth):
-    """Decorate a bound wsgi callable taking args from
-    ``wsgiorg.routing_args``
-    ::
+    """
+    Decorate a bound wsgi callable taking args from ``wsgiorg.routing_args``.
+
+    .. code-block:: python
 
         class App:
             @opliant
