@@ -100,6 +100,9 @@ accesskeysuffixes = (".accesskey", ".accessKey", ".akey")
 ending in :attr:`.labelsuffixes` into accelerator notation"""
 
 
+end_entity_re = re.compile(rb"[\"']\s*>")
+
+
 def quoteforandroid(source):
     """Escapes a line for Android DTD files."""
     # Replace "'" character with the \u0027 escape. Other possible replaces are
@@ -207,11 +210,9 @@ def removeinvalidamps(name, value):
                 continue
             invalid_amps.append(amppos - 1)
     if len(invalid_amps) > 0:
-        warnings.warn("invalid ampersands in dtd entity %s" % (name))
-        adjustment = 0
-        for amppos in invalid_amps:
+        warnings.warn(f"invalid ampersands in dtd entity {name}")
+        for adjustment, amppos in enumerate(invalid_amps):
             value = value[: amppos - adjustment] + value[amppos - adjustment + 1 :]
-            adjustment += 1
     return value
 
 
@@ -290,9 +291,7 @@ class dtdunit(base.TranslationUnit):
         return self.entity is None
 
     def istranslatable(self):
-        if getattr(self, "entityparameter", None) == "SYSTEM" or self.isblank():
-            return False
-        return True
+        return getattr(self, "entityparameter", None) != "SYSTEM" and not self.isblank()
 
     def parse(self, dtdsrc):
         """Read the first dtd element from the source code into this object, return linesprocessed."""
@@ -446,7 +445,7 @@ class dtdunit(base.TranslationUnit):
                     e = 0
                     if not line:
                         continue
-                    if line[0] in ('"', "'"):
+                    if line[0] in {'"', "'"}:
                         self.entitypart = "definition"
                         self.entityhelp = (e, line[e])
                         self.instring = False
@@ -479,7 +478,7 @@ class dtdunit(base.TranslationUnit):
                         )
                     else:
                         raise ValueError(
-                            "Unexpected quote character... %r" % (self.entityhelp[1])
+                            f"Unexpected quote character... {self.entityhelp[1]!r}"
                         )
                     # for any following lines, start at the beginning of the line. remember the quote character
                     self.entityhelp = (0, self.entityhelp[1])
@@ -561,7 +560,7 @@ class dtdfile(base.TranslationStore):
                     break
                 if lines[end].find(b"<!ENTITY") > -1:
                     foundentity = True
-                if foundentity and re.match(rb"[\"']\s*>", lines[end]):
+                if foundentity and end_entity_re.match(lines[end]):
                     end += 1
                     break
                 end += 1
@@ -592,7 +591,7 @@ class dtdfile(base.TranslationStore):
             out.write(unit_str)
             content += unit_str
         if not self._valid_store(content):
-            warnings.warn("DTD file '%s' does not validate" % self.filename)
+            warnings.warn(f"DTD file '{self.filename}' does not validate")
             out.truncate(0)
 
     def _valid_store(self, content):
@@ -611,6 +610,6 @@ class dtdfile(base.TranslationStore):
             try:
                 etree.DTD(BytesIO(_input))
             except etree.DTDParseError as e:
-                warnings.warn("DTD parse error: %s" % e.error_log)
+                warnings.warn(f"DTD parse error: {e.error_log}")
                 return False
         return True
