@@ -25,7 +25,8 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 
-"""Module for parsing Gettext .mo files for translation.
+"""
+Module for parsing Gettext .mo files for translation.
 
 The coding of .mo files was produced from `Gettext documentation
 <http://www.gnu.org/software/gettext/manual/gettext.html#MO-Files>`_,
@@ -50,7 +51,7 @@ POT_HEADER = re.compile(r"^POT-Creation-Date:.*(\n|$)", re.IGNORECASE | re.MULTI
 
 
 def mounpack(filename="messages.mo"):
-    """Helper to unpack Gettext MO files into a Python string"""
+    """Helper to unpack Gettext MO files into a Python string."""
     with open(filename, "rb") as fh:
         s = fh.read()
         print("\\x%02x" * len(s) % tuple(map(ord, s)))
@@ -71,12 +72,12 @@ def hashpjw(str_param):
     for s in str_param:
         if not s:
             break
-        hval = hval << 4
+        hval <<= 4
         hval += s
         g = hval & 0xF << (HASHWORDBITS - 4)
         if g != 0:
-            hval = hval ^ (g >> (HASHWORDBITS - 8))
-            hval = hval ^ g
+            hval ^= g >> HASHWORDBITS - 8
+            hval ^= g
     return hval
 
 
@@ -114,7 +115,7 @@ class mounit(base.TranslationUnit):
         super().__init__(source)
 
     def getcontext(self):
-        """Get the message context"""
+        """Get the message context."""
         # Still need to handle KDE comments
         if self.msgctxt is None:
             return None
@@ -124,11 +125,11 @@ class mounit(base.TranslationUnit):
         self.msgctxt = [context]
 
     def isheader(self):
-        """Is this a header entry?"""
+        """Is this a header entry?."""
         return self.source == ""
 
     def istranslatable(self):
-        """Is this message translateable?"""
+        """Is this message translateable?."""
         return bool(self.source)
 
 
@@ -148,7 +149,7 @@ class mofile(poheader.poheader, base.TranslationStore):
             self.parsestring(inputfile)
 
     def serialize(self, out):
-        """Output a string representation of the MO data file"""
+        """Output a string representation of the MO data file."""
         # check the header of this file for the copyright note of this function
 
         def add_to_hash_table(string, i):
@@ -157,7 +158,7 @@ class mofile(poheader.poheader, base.TranslationStore):
             increment = 1 + (hash_value % (hash_size - 2))
             while hash_table[hash_cursor] != 0:
                 hash_cursor += increment
-                hash_cursor = hash_cursor % hash_size
+                hash_cursor %= hash_size
             hash_table[hash_cursor] = i + 1
 
         def lst_encode(lst, join_char=b""):
@@ -216,8 +217,8 @@ class mofile(poheader.poheader, base.TranslationStore):
         # The string table first has the list of keys, then the list of values.
         # Each entry has first the size of the string, then the file offset.
         for o1, l1, o2, l2 in offsets:
-            koffsets = koffsets + [l1, o1 + keystart]
-            voffsets = voffsets + [l2, o2 + valuestart]
+            koffsets = [*koffsets, l1, o1 + keystart]
+            voffsets = [*voffsets, l2, o2 + valuestart]
         offsets = koffsets + voffsets
         out.write(
             struct.pack(
@@ -239,7 +240,7 @@ class mofile(poheader.poheader, base.TranslationStore):
             out.write(strs)
 
     def parse(self, input):
-        """parses the given file or file source string"""
+        """Parses the given file or file source string."""
         if hasattr(input, "name"):
             self.filename = input.name
         elif not getattr(self, "filename", ""):
@@ -265,7 +266,7 @@ class mofile(poheader.poheader, base.TranslationStore):
             startvalue,
             _sizehash,
             _offsethash,
-        ) = struct.unpack("%sLHHiiiii" % endian, input[: (7 * 4)])
+        ) = struct.unpack(f"{endian}LHHiiiii", input[: (7 * 4)])
         if version_maj >= 1:
             raise base.ParseError(
                 """Unable to process version %d.%d MO files"""
@@ -275,10 +276,10 @@ class mofile(poheader.poheader, base.TranslationStore):
             nextkey = startkey + (i * 2 * 4)
             nextvalue = startvalue + (i * 2 * 4)
             klength, koffset = struct.unpack(
-                "%sii" % endian, input[nextkey : nextkey + (2 * 4)]
+                f"{endian}ii", input[nextkey : nextkey + (2 * 4)]
             )
             vlength, voffset = struct.unpack(
-                "%sii" % endian, input[nextvalue : nextvalue + (2 * 4)]
+                f"{endian}ii", input[nextvalue : nextvalue + (2 * 4)]
             )
             source = input[koffset : koffset + klength]
             context = None
