@@ -16,7 +16,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Classes that hold units of PHP localisation files :class:`phpunit` or
+"""
+Classes that hold units of PHP localisation files :class:`phpunit` or
 entire files :class:`phpfile`. These files are used in translating many
 PHP based applications.
 
@@ -80,12 +81,12 @@ def wrap_production(func):
     def prod(n):
         func(n)
         if isinstance(n[0], Node):
-            startpos = min([getattr(i, "lexpos", 0) for i in n.slice[1:]])
-            endpos = max([getattr(i, "endlexpos", 0) for i in n.slice[1:]])
+            startpos = min(getattr(i, "lexpos", 0) for i in n.slice[1:])
+            endpos = max(getattr(i, "endlexpos", 0) for i in n.slice[1:])
             n[0].lexpositions = startpos, endpos
         elif isinstance(n[0], list) and n[0] and isinstance(n[0][-1], ArrayElement):
-            startpos = min([getattr(i, "lexpos", 0) for i in n.slice[1:]])
-            endpos = max([getattr(i, "endlexpos", 0) for i in n.slice[1:]])
+            startpos = min(getattr(i, "lexpos", 0) for i in n.slice[1:])
+            endpos = max(getattr(i, "endlexpos", 0) for i in n.slice[1:])
             n[0][-1].lexpositions = startpos, endpos
 
     return prod
@@ -105,24 +106,25 @@ class PHPLexer(FilteredLexer):
         return token
 
     def extract_comments(self, end):
-        """Extract comments related to given parser positions.
+        """
+        Extract comments related to given parser positions.
 
         Must be called sequentially for consequent statements.
         """
         comments = []
         # Process all tokens to end of statement
         while self.tokens[self.pos].lexpos < end:
-            if self.tokens[self.pos].type in ("COMMENT", "DOC_COMMENT"):
+            if self.tokens[self.pos].type in {"COMMENT", "DOC_COMMENT"}:
                 comments.append(self.tokens[self.pos].value.strip())
             self.pos += 1
         # Skip end of statement
         self.pos += 1
         # Proceed comments till newline
         length = len(self.tokens)
-        while self.pos < length and self.tokens[self.pos].type in (
+        while self.pos < length and self.tokens[self.pos].type in {
             "COMMENT",
             "WHITESPACE",
-        ):
+        }:
             token_type = self.tokens[self.pos].type
             token_value = self.tokens[self.pos].value
             self.pos += 1
@@ -150,11 +152,11 @@ class PHPLexer(FilteredLexer):
     def extract_quote(self):
         """Extract quote style."""
         pos = max(self.pos, self.codepos)
-        while self.tokens[pos].type not in (
+        while self.tokens[pos].type not in {
             "QUOTE",
             "CONSTANT_ENCAPSED_STRING",
             "START_NOWDOC",
-        ):
+        }:
             pos += 1
         if self.tokens[pos].type == "QUOTE":
             return '"'
@@ -162,7 +164,7 @@ class PHPLexer(FilteredLexer):
 
     def extract_array(self):
         pos = max(self.pos, self.codepos)
-        while self.tokens[pos].type not in ("ARRAY", "LBRACKET"):
+        while self.tokens[pos].type not in {"ARRAY", "LBRACKET"}:
             pos += 1
         self.codepos = pos
         if self.tokens[pos].type == "ARRAY":
@@ -171,7 +173,8 @@ class PHPLexer(FilteredLexer):
 
 
 def phpencode(text, quotechar="'"):
-    """Convert Python string to PHP escaping.
+    """
+    Convert Python string to PHP escaping.
 
     The encoding is implemented for
     `'single quote' <http://www.php.net/manual/en/language.types.string.php#language.types.string.syntax.single>`_
@@ -201,23 +204,20 @@ def phpencode(text, quotechar="'"):
         for a, b in escapes:
             text = text.replace(a, b)
         return text
-    else:
-        return text.replace("%s" % quotechar, "\\%s" % quotechar)
+    return text.replace(f"{quotechar}", f"\\{quotechar}")
 
 
 def phpdecode(text, quotechar="'"):
     """Convert PHP escaped string to a Python string."""
-
     escape_encoding = "unicode_escape"
 
     def decode_octal_hex(match):
-        r"""decode Octal \NNN and Hex values"""
+        r"""Decode Octal \NNN and Hex values."""
         if "octal" in match.groupdict():
             return match.groupdict()["octal"].encode("latin-1").decode(escape_encoding)
-        elif "hex" in match.groupdict():
+        if "hex" in match.groupdict():
             return match.groupdict()["hex"].encode("latin-1").decode(escape_encoding)
-        else:
-            return match.group
+        return match.group
 
     if not text:
         return text
@@ -236,10 +236,8 @@ def phpdecode(text, quotechar="'"):
         for a, b in escapes:
             text = text.replace(a, b)
         text = re.sub(r"(?P<octal>\\[0-7]{1,3})", decode_octal_hex, text)
-        text = re.sub(r"(?P<hex>\\x[0-9A-Fa-f]{1,2})", decode_octal_hex, text)
-        return text
-    else:
-        return text.replace("\\'", "'").replace("\\\\", "\\")
+        return re.sub(r"(?P<hex>\\x[0-9A-Fa-f]{1,2})", decode_octal_hex, text)
+    return text.replace("\\'", "'").replace("\\\\", "\\")
 
 
 class phpunit(base.TranslationUnit):
@@ -292,12 +290,12 @@ class phpunit(base.TranslationUnit):
         else:
             fmt = "{0} = {1}{2}{1};\n"
         out = fmt.format(
-            name if name else self.name,
+            name or self.name,
             self.escape_type,
             phpencode(self.get_raw_value(), self.escape_type),
         )
         joiner = "\n" + indent
-        return indent + joiner.join(self._comments + [out])
+        return indent + joiner.join([*self._comments, out])
 
     def addlocation(self, location):
         self.name = location
@@ -306,7 +304,7 @@ class phpunit(base.TranslationUnit):
         return [self.name]
 
     def addnote(self, text, origin=None, position="append"):
-        if origin in ["programmer", "developer", "source code", None]:
+        if origin in {"programmer", "developer", "source code", None}:
             if position == "append":
                 self._comments.append(text)
             else:
@@ -315,10 +313,9 @@ class phpunit(base.TranslationUnit):
             super().addnote(text, origin=origin, position=position)
 
     def getnotes(self, origin=None):
-        if origin in ["programmer", "developer", "source code", None]:
+        if origin in {"programmer", "developer", "source code", None}:
             return "\n".join(self._comments)
-        else:
-            return super().getnotes(origin)
+        return super().getnotes(origin)
 
     def removenotes(self, origin=None):
         self._comments = []
@@ -452,7 +449,7 @@ class phpfile(base.TranslationStore):
         def concatenate(item):
             if isinstance(item, str):
                 return item
-            elif isinstance(item, Variable):
+            if isinstance(item, Variable):
                 return item.name
             assert isinstance(item, BinaryOp)
             return concatenate(item.left) + concatenate(item.right)
@@ -476,25 +473,7 @@ class phpfile(base.TranslationStore):
                         lexer.extract_comments(item.lexpositions[1]),
                     )
             elif isinstance(item, Assignment):
-                if isinstance(item.node, ArrayOffset):
-                    name = lexer.extract_name("EQUALS", *item.lexpositions)
-                    if isinstance(item.expr, Array):
-                        handle_array(name, item.expr.nodes, lexer)
-                    elif isinstance(item.expr, str):
-                        self.create_and_add_unit(
-                            name,
-                            item.expr,
-                            lexer.extract_quote(),
-                            lexer.extract_comments(item.lexpositions[1]),
-                        )
-                    elif isinstance(item.expr, BinaryOp) and item.expr.op == ".":
-                        self.create_and_add_unit(
-                            name,
-                            concatenate(item.expr),
-                            lexer.extract_quote(),
-                            lexer.extract_comments(item.lexpositions[1]),
-                        )
-                elif isinstance(item.node, Variable):
+                if isinstance(item.node, (ArrayOffset, Variable)):
                     name = lexer.extract_name("EQUALS", *item.lexpositions)
                     if isinstance(item.expr, Array):
                         handle_array(name, item.expr.nodes, lexer)
