@@ -16,16 +16,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""module for handling TBX glossary files"""
+"""module for handling TBX glossary files."""
 
 from lxml import etree
 
-from translate.misc.xml_helpers import getXMLspace, setXMLlang
+from translate.misc.xml_helpers import (
+    getXMLspace,
+    safely_set_text,
+    setXMLlang,
+)
 from translate.storage import lisa
 
 
 class tbxunit(lisa.LISAunit):
-    """A single term in the TBX file.  Provisional work is done to make several
+    """
+    A single term in the TBX file.  Provisional work is done to make several
     languages possible.
     """
 
@@ -34,14 +39,14 @@ class tbxunit(lisa.LISAunit):
     textNode = "term"
 
     def createlanguageNode(self, lang, text, purpose):
-        """returns a langset xml Element setup with given parameters"""
+        """Returns a langset xml Element setup with given parameters."""
         langset = etree.Element(self.languageNode)
         setXMLlang(langset, lang)
         tig = etree.SubElement(langset, "tig")  # or ntig with termGrp inside
         term = etree.SubElement(tig, self.textNode)
         # probably not what we want:
         # lisa.setXMLspace(term, "preserve")
-        term.text = text
+        safely_set_text(term, text)
         return langset
 
     def getid(self):
@@ -54,7 +59,7 @@ class tbxunit(lisa.LISAunit):
     def _get_origin_element(self, origin: str):
         if origin == "pos":
             return self.namespaced("termNote")
-        elif origin == "definition":
+        if origin == "definition":
             return self.namespaced("descrip")
         return self.namespaced("note")
 
@@ -65,7 +70,7 @@ class tbxunit(lisa.LISAunit):
             self.xmlelement.remove(note)
 
     def addnote(self, text, origin=None, position="append"):
-        """Add a note specifically in a "note" tag"""
+        """Add a note specifically in a "note" tag."""
         if position != "append":
             self.removenotes(origin=origin)
 
@@ -74,12 +79,13 @@ class tbxunit(lisa.LISAunit):
         if not text:
             return
         note = etree.SubElement(self.xmlelement, self._get_origin_element(origin))
-        note.text = text
-        if origin and origin not in ("pos", "definition"):
+        safely_set_text(note, text)
+        if origin and origin not in {"pos", "definition"}:
             note.set("from", origin)
 
     def _getnotelist(self, origin=None):
-        """Returns the text from notes matching ``origin`` or all notes.
+        """
+        Returns the text from notes matching ``origin`` or all notes.
 
         :param origin: The origin of the note (or note type)
         :type origin: String
@@ -93,7 +99,7 @@ class tbxunit(lisa.LISAunit):
         initial_list = [
             lisa.getText(note, getXMLspace(self.xmlelement, self._default_xml_space))
             for note in note_nodes
-            if origin in ("pos", "definition", None) or note.get("from") == origin
+            if origin in {"pos", "definition", None} or note.get("from") == origin
         ]
 
         # Remove duplicate entries from list:
