@@ -19,8 +19,10 @@
 
 """Module to provide a translation memory database."""
 
+import contextlib
 import logging
 import math
+import operator
 import re
 import threading
 import time
@@ -77,8 +79,7 @@ class TMDB:
     cursor = property(lambda self: self._get_connection(1))
 
     def init_database(self):
-        """creates database tables and indices"""
-
+        """Creates database tables and indices."""
         script = """
 CREATE TABLE IF NOT EXISTS sources (
        sid INTEGER PRIMARY KEY AUTOINCREMENT,
@@ -114,8 +115,7 @@ CREATE UNIQUE INDEX IF NOT EXISTS targets_uniq_idx ON targets (sid, text, lang);
             raise
 
     def init_fulltext(self):
-        """detects if fts3 fulltext indexing module exists, initializes fulltext table if it does"""
-
+        """Detects if fts3 fulltext indexing module exists, initializes fulltext table if it does."""
         # HACKISH: no better way to detect fts3 support except trying to
         # construct a dummy table?!
         try:
@@ -163,9 +163,9 @@ END;
             logging.debug("created fulltext triggers")
             self.fulltext = True
 
-        except dbapi2.OperationalError as e:
+        except dbapi2.OperationalError:
             self.fulltext = False
-            logging.debug("failed to initialize fts3 support: " + str(e))
+            logging.exception("failed to initialize fts3 support")
             script = """
 DROP TRIGGER IF EXISTS sources_insert_trig;
 DROP TRIGGER IF EXISTS sources_update_trig;
@@ -174,8 +174,9 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
             self.cursor.executescript(script)
 
     def preload_db(self):
-        """ugly hack to force caching of sqlite db file in memory for improved
-        performance
+        """
+        ugly hack to force caching of sqlite db file in memory for improved
+        performance.
         """
         if self.fulltext:
             query = """SELECT COUNT(*) FROM sources s JOIN fulltext f ON s.sid = f.docid JOIN targets t on s.sid = t.sid"""
@@ -187,7 +188,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         return numrows
 
     def add_unit(self, unit, source_lang=None, target_lang=None, commit=True):
-        """inserts unit in the database"""
+        """Inserts unit in the database."""
         # TODO: is that really the best way to handle unspecified
         # source and target languages? what about conflicts between
         # unit attributes and passed arguments
@@ -209,7 +210,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         self.add_dict(unitdict, source_lang, target_lang, commit)
 
     def add_dict(self, unit, source_lang, target_lang, commit=True):
-        """inserts units represented as dictionaries in database"""
+        """Inserts units represented as dictionaries in database."""
         source_lang = data.normalize_code(source_lang)
         target_lang = data.normalize_code(target_lang)
         try:
@@ -227,16 +228,14 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
                 )
                 sid = self.cursor.fetchone()
                 (sid,) = sid
-            try:
+            # target string already exists in db, do nothing
+            with contextlib.suppress(dbapi2.IntegrityError):
                 # FIXME: get time info from translation store
                 # FIXME: do we need so store target length?
                 self.cursor.execute(
                     "INSERT INTO targets (sid, text, lang, time) VALUES (?, ?, ?, ?)",
                     (sid, unit["target"], target_lang, int(time.time())),
                 )
-            except dbapi2.IntegrityError:
-                # target string already exists in db, do nothing
-                pass
 
             if commit:
                 self.connection.commit()
@@ -246,7 +245,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
             raise
 
     def add_store(self, store, source_lang, target_lang, commit=True):
-        """insert all units in store in database"""
+        """Insert all units in store in database."""
         count = 0
         for unit in store.units:
             if unit.istranslatable() and unit.istranslated():
@@ -257,8 +256,9 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         return count
 
     def add_list(self, units, source_lang, target_lang, commit=True):
-        """insert all units in list into the database, units are represented as
-        dictionaries
+        """
+        insert all units in list into the database, units are represented as
+        dictionaries.
         """
         count = 0
         for unit in units:
@@ -269,7 +269,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         return count
 
     def translate_unit(self, unit_source, source_langs, target_langs):
-        """return TM suggestions for unit_source"""
+        """Return TM suggestions for unit_source."""
         if isinstance(source_langs, list):
             source_langs = [data.normalize_code(lang) for lang in source_langs]
             source_langs = ",".join(source_langs)
@@ -319,7 +319,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
                         "quality": quality,
                     }
                 )
-        results.sort(key=lambda match: match["quality"], reverse=True)
+        results.sort(key=operator.itemgetter("quality"), reverse=True)
         results = results[: self.max_candidates]
         logging.debug("results: %s", str(results))
         return results
